/* Copyright (C) 1991, 1992, 1993 Free Software Foundation, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
                
#ifndef LIBDEFS_H
 #define LIBDEFS_H
 #include <libdefs.h>
#endif
#include <mhash.h>
#include <bzero.h>
#include <swap.h>
#include <crc32.h>
#include <md5.h>
#include <sha1.h>

#define MAX_THREADS 256 /* After that pointers are being reused */

/* I think now is re-entrant */
static int check_pointer[MAX_THREADS]={0};
static int used_all_threads=0;

#ifdef HAVE_LIBPTHREAD
# include <pthread.h>
static int cur_thread=0;
pthread_mutex_t counter_cur_thread = PTHREAD_MUTEX_INITIALIZER;
#else
static int cur_thread=0; /* No threads */
#endif

static MD5_CTX *md5; /* For md5 */
static SHA1_CTX *sha1; /* For sha1 */
static word32 *crc; /* For crc32 */

/* hold the key and the algorithm */
static int *algorithm_given=NULL;

int mhash_get_block_size(int algorithm)
{
  switch(algorithm) {
  
   case CRC32:
      return 4;
      break;

   case MD5:
      return 16;
      break;

   case SHA1:
      return 20;
      break;

   default:
      return 0;
  }


}



int init_mhash_int(const int algorithm, int thread)
{

 if (used_all_threads==0) {
  crc=realloc(crc, (thread+1)*sizeof(word32));
  md5=realloc(md5, (thread+1)*sizeof(MD5_CTX));
  sha1=realloc(sha1, (thread+1)*sizeof(SHA1_CTX));
  algorithm_given=realloc(algorithm_given, (thread+1)*sizeof(int));
 }
 
  algorithm_given[thread]=algorithm;
  check_pointer[thread]=1;
  
   switch(algorithm) {
  
   case CRC32:
      clear_crc32(&crc[thread]);
      return thread;
      break;

   case MD5:
      MD5Init(&md5[thread]);
      return thread;
      break;

   case SHA1:
      SHA1Init(&sha1[thread]);
      return thread;
      break;
      
   default:
      return -1;
  }

}






/* plaintext should be in block's size */
int mhash(int thread, void *plaintext, int lenoftext)
{

  switch(algorithm_given[thread]) {

   case CRC32:
      crc32(&crc[thread], plaintext, lenoftext);
      return 0;
      break;

   case MD5:
      MD5Update(&md5[thread], plaintext, lenoftext);
      return 0;
      break;

   case SHA1:
      SHA1Update(&sha1[thread], plaintext, lenoftext);
      return 0;
      break;

   default:
      return 0;
  }

}


void* end_mhash(int td)
{
check_pointer[td]=0;

   switch(algorithm_given[td]) {
  
   case CRC32:
      return get_crc32(&crc[td]);
      break;

   case MD5:
      return MD5Final(&md5[td]);
      break;

   case SHA1:
      return SHA1Final(&sha1[td]);
      break;

   default:
      return NULL;
  }

return 0;
}



int init_mhash(const int algorithm)
{

int x;

#ifdef HAVE_LIBPTHREAD
pthread_mutex_lock(&counter_cur_thread);
#endif 


  if (cur_thread == MAX_THREADS) {
     cur_thread=0;
     used_all_threads=1;
  }  

  while (check_pointer[cur_thread]==1 && cur_thread < MAX_THREADS) {
    cur_thread++;
  }  
  
  if (cur_thread >= MAX_THREADS) {
     #ifdef HAVE_LIBPTHREAD
     pthread_mutex_unlock(&counter_cur_thread);
     #endif
     return -1;
  }

  x=init_mhash_int(algorithm, cur_thread);

 if (x<0) {
   end_mhash(cur_thread);
   #ifdef HAVE_LIBPTHREAD
   pthread_mutex_unlock(&counter_cur_thread);
   #endif
   return -1;
 }
 else {
   #ifdef HAVE_LIBPTHREAD
   pthread_mutex_unlock(&counter_cur_thread);
   #endif
   return x;
 }

}


